<?php
if (!function_exists('random_bytes')) {
    /**
     * EXAMPLE CODE -- DO NOT USE THIS IN YOUR PROJECTS! 
     * You want this instead:
     * https://github.com/paragonie/random_compat
     * 
     * PHP 5.2.0 - 5.6.x way to implement random_bytes()
     * 
     * @param int $bytes
     * @return string
     */
    function random_bytes($bytes)
    {
        if (!is_int($bytes) || $bytes < 1) {
             throw new InvalidArgumentException('random_bytes() expects a positive integer');
        }
        $buf = '';
        // See PHP bug #55169 for why 5.3.7 is required
        if (
            function_exists('mcrypt_create_iv') &&
            version_compare(PHP_VERSION, '5.3.7') >= 0
        ) {
            $buf = mcrypt_create_iv($bytes, MCRYPT_DEV_URANDOM);
            if ($buf !== false) {
                return $buf;
            }
        }
        
        /**
         * Use /dev/urandom for random numbers
         * 
         * @ref http://sockpuppet.org/blog/2014/02/25/safely-generate-random-numbers
         */
        if (@is_readable('/dev/urandom')) {
            $fp = fopen('/dev/urandom', 'rb');
            if ($fp !== false) {
                $streamset = stream_set_read_buffer($fp, 0);
                if ($streamset === 0) {
                    $remaining = $bytes;
                    do {
                        $read = fread($fp, $remaining); 
                        if ($read === false) {
                            // We cannot safely read from urandom.
                            $buf = false;
                            break;
                        }
                        // Decrease the number of bytes returned from remaining
                        $remaining -= strlen($read);
                        $buf .= $read;
                    } while ($remaining > 0);
                    if ($buf !== false) {
                        return $buf;
                    }
                }
            }
        }
        /**
         * Windows with PHP < 5.3.0 will not have the function
         * openssl_random_pseudo_bytes() available, so let's use
         * CAPICOM to work around this deficiency.
         */
         if (class_exists('COM', false)) {
             try {
                 if ($buf === false) {
                     $buf = ''; // Make it a string, not false
                 }
                $util = new COM('CAPICOM.Utilities.1');
                $execs = 0;
                /**
                 * Let's not let it loop forever. If we run N times and fail to
                 * get N bytes of random data, then CAPICOM has failed us.
                 */
                do {
                    $buf .= base64_decode($util->GetRandom($bytes, 0));
                    if (strlen($buf) >= $bytes) { 
                        return substr($buf, 0, $bytes);
                    }
                    ++$execs;
                } while ($execs < $bytes);
            } catch (Exception $e) {
                unset($e); // Let's not let CAPICOM errors kill our app
            }
        }
        if (function_exists('openssl_random_pseudo_bytes')) {
            $secure = true;
            $buf = openssl_random_pseudo_bytes($bytes, $secure);
            if ($buf !== false && $secure) {
                return $buf;
            }
        }
        /**
         * We have reached the point of no return. Throw an exception.
         */
        throw new Exception('Better for PRNG failures to terminate than propagate');
    }
}